from unittest.mock import patch

import pytest

from gso.products import Router
from test.workflows import assert_complete, extract_state, run_workflow


@pytest.fixture
def router_termination_input_form_data(site_subscription_factory, faker):
    return {"tt_number": faker.tt_number(), "remove_configuration": True, "clean_up_ipam": True}


@pytest.mark.workflow
@patch("gso.workflows.router.terminate_router.NetboxClient.delete_device")
@patch("gso.workflows.router.terminate_router.infoblox.delete_host_by_ip")
@patch("gso.workflows.router.terminate_router.infoblox.delete_network")
def test_terminate_router_success(
    mock_delete_network,
    mock_delete_host_by_ip,
    mock_delete_device,
    router_termination_input_form_data,
    router_subscription_factory,
    faker,
):
    #  Set up active subscription in database
    product_id = router_subscription_factory()

    #  Run workflow
    initial_router_data = [{"subscription_id": product_id}, router_termination_input_form_data]
    result, process_stat, step_log = run_workflow("terminate_router", initial_router_data)
    assert_complete(result)

    state = extract_state(result)
    subscription_id = state["subscription_id"]
    subscription = Router.from_subscription(subscription_id)

    assert "terminated" == subscription.status
    assert mock_delete_network.call_count == 3
    assert mock_delete_device.call_count == 1
    assert mock_delete_host_by_ip.call_count == 1
